/**
 * Belief (Bayesian) Network Node
 * Creation date: (10/23/00 10:19:48 AM)
 * @author: Todd Neller
 */


class BNNode {
    /** list of BNNodes */
    static java.util.Vector nodes = new java.util.Vector();
    /** variable name */
    String name;
    /** variable value */
    boolean value;
    /** node parents */
    BNNode[] parents;
    /** node children */
    BNNode[] children;
    /** conditional probability table indexed by
        parents[0].value?1:0 * 2^(p-1)
        + parents[1].value?1:0 * 2^(p-2)
        + ...
        + parents[p-1].value?1:0 * 1
    */
    double[] cpt;
    /** whether or not node is an evidence node */
    boolean isEvidence = false;


/**
 * BNNode constructor.
 */
public BNNode() {
    super();
}


/**
 * Compute the children from the parents.  A node with no children will have an empty array.
 * Creation date: (10/23/00 4:52:24 PM)
 */
static void computeChildren() {
    BNNode node, node2;
    java.util.Vector childrenVector = new java.util.Vector();
    // For each node, build an array of children by checking which nodes have it as a parent.
    for (int i = 0; i < BNNode.nodes.size(); i++) {
        node = (BNNode) BNNode.nodes.elementAt(i);
        childrenVector.removeAllElements();
        for (int j = 0; j < BNNode.nodes.size(); j++) {
            if (i == j)
                continue;
            node2 = (BNNode) BNNode.nodes.elementAt(j);
            for (int k = 0; node2.parents != null && k < node2.parents.length; k++)
                if (node2.parents[k] == node)
                    childrenVector.addElement(node2);
        }
        node.children = new BNNode[childrenVector.size()];
        node.children = (BNNode[]) childrenVector.toArray(node.children);
    }
}


/**
 * Returns Conditional Probability Table (CPT) entry corresponding to current parent node values.
 * To figure the mapping from parent values to table entry, imagine parent values as digits of a binary number.
 * (The first/last parent is the most/least significant digit.)
 * Convert this binary number to an integer, and you have the index to the CPT.
 * Example: P(+a|-b -c) = 0.1   P(+a|-b +c) = 0.2   P(+a|+b -c) = 0.3   P(+a|+b +c) = 0.4
 *                0  0                0  1                1  0                1  1
 *                  0                   1                   2                   3
 * cpt == {0.1, 0.2, 0.3, 0.4}
 * Creation date: (10/23/00 1:34:39 PM)
 * @return double
 */
public double cptLookup() {
    int entry = 0;
    int powerOfTwo = 1;
    for (int i = parents.length - 1; i >= 0; i--) {
        if (parents[i].value)
            entry += powerOfTwo;
        powerOfTwo *= 2;
    }
    /*System.out.println("cptLookup for " + name);
    for (int i = 0; i < parents.length; i++) {
        System.out.println((parents[i]).name + " " + (parents[i]).value + " ");
    }
    System.out.println("Entry " + entry + "=" + cpt[entry]);*/
    return cpt[entry];
}


/**
 * Print all information about the BN.
 * Creation date: (10/23/00 5:23:17 PM)
 */
public static void printBN() {
    for (int i = 0; i < BNNode.nodes.size(); i++)
        System.out.println((BNNode) BNNode.nodes.elementAt(i));
}


/**
 * Return String with information about node.
 * Creation date: (10/23/00 12:07:32 PM)
 * @return java.lang.String
 */
public String toString() {
    String s = "BNNode " + name + ":\n";
    if (isEvidence)
        s += "  EVIDENCE\n";
    s += "  value " + value + "\n";
    s += "  parents:";
    if (parents.length == 0)
        s += " (none)";
    else
        for (int i = 0; i < parents.length; i++)
            s += " " + parents[i].name;
    s += "\n  children:";
    if (children.length == 0)
        s += " (none)";
    else
        for (int i = 0; i < children.length; i++)
            s += " " + children[i].name;
    s += "\n  CPT:";
    for (int i = 0; i < cpt.length; i++)
        s += " " + cpt[i];
    return s;
}
}





